part of '../../simple_type.dart';

/*
 * Copyright 2020 Pawan Kumar. All rights reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/// Extension Methods & Widgets for the numbers
extension NumExtension on num {
  ///
  /// Use this extension method to get fixed values after decimal with defined length.
  ///
  String toDoubleStringAsFixed({int digit = 2}) =>
      toStringAsFixed(truncateToDouble() == this ? 0 : digit);

  ///
  /// Use this extension method to get precised values after decimal with defined length.
  ///
  String toDoubleStringAsPrecised({int length = 4}) {
    return ((this * pow(10, length)).truncateToDouble() / pow(10, length))
        .toString();
  }

  ///
  /// Check if the number is null
  ///
  bool get isNull => this == null;

  ///
  /// Check if the number is not null
  ///
  bool get isNotNull => this != null;

  /// Returns if the number is even
  bool get isEven => this % 2 == 0;

  /// Returns if the number is odd
  bool get isOdd => this % 2 != 0;

  /// Returns if the number is positive
  bool get isPositive => this > 0;

  /// Returns if the number is negative
  bool get isNegative => this < 0;

  double get a => St._apply(toDouble());

  double get wsz => sc.wsz(toDouble());

  double get sz => sc.sz(toDouble());

  double get hsz => sc.hsz(toDouble());

  double get degrees => toDouble() * pi / 180;

  Radius circularRadius([bool sc = true]) =>
      Radius.circular(St._apply(toDouble(), sc));

  BorderRadius circularBorderRadius([bool sc = true]) =>
      BorderRadius.circular(St._apply(toDouble(), sc));

  /// Format numeric currency
  String get numCurrency =>
      intl.NumberFormat.currency(customPattern: '#,##0.00').format(this);

  /// Format numeric currency with provided locale
  String numCurrencyWithLocale({String locale = 'en_US'}) =>
      intl.NumberFormat.currency(
        locale: locale,
      ).format(this);

  //! Widgets

  Tween<double> tweenTo(double end) =>
      Tween<double>(begin: toDouble(), end: end);

  Spacer get spacer => Spacer(flex: toInt());

  ///
  ///Converts the number into a [SizedBox] with the width as that number.
  ///
  Widget wBox([bool sc = true]) => SizedBox(width: St._apply(toDouble(), sc));

  ///
  ///Converts the number into a [SizedBox] with the height as that number.
  ///
  Widget hBox([bool sc = true]) => SizedBox(height: St._apply(toDouble(), sc));

  ///
  ///Converts the number into a [SizedBox] with the width & height as that number.
  ///
  Widget sqBox([bool sc = true]) => SizedBox(
      height: St._apply(toDouble(), sc), width: St._apply(toDouble(), sc));

  /// Get Text Widget for the String
  Text get text => Text(toString());
}

/// Extension method for integers.
extension IntExtension on int {
  int get random => Random().nextInt(this);

  List<Widget> builder(Widget Function(int index) element) {
    final List<Widget> widgets = [];
    for (var i = 0; i < this; i++) {
      widgets.add(element(i));
    }
    return widgets;
  }
}
